/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.history;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.core.history.provider.FileHistory;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.BazaarRevision.Prefix;
import org.vcs.bazaar.client.BazaarRevisionRange;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.IBazaarRevisionSpec;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;

/**
 *
 * @author Guillermo Gonzalez
 *
 */
public class BazaarFileHistory extends FileHistory {
	private IResource resource;

	protected SortedMap<BazaarRevision, BazaarFileRevision> revisionMap;
	protected BazaarRevision lastLoadedRevision;
	
	protected BazaarFileHistory() {
		revisionMap = new TreeMap<BazaarRevision, BazaarFileRevision>(new Comparator<BazaarRevision>() {
			public int compare(BazaarRevision rev1, BazaarRevision rev2) {
				return rev1.compareTo(rev2);
			}
		});
	}

	public BazaarFileHistory(IResource resource) {
		this();
		this.resource = resource;
	}

	public BazaarFileHistory(List<IBazaarLogMessage> logMessages) {
		this();
		addRevisions(logMessages);
	}

	public IFileRevision[] getContributors(IFileRevision revision) {
		final BazaarFileRevision bazaarRevision = (BazaarFileRevision) revision;
		final List<IBazaarLogMessage> merged = bazaarRevision.getIBazaarLogMessage().getMerged();
		if (merged != null && merged.size() > 0) {
			final List<IFileRevision> revisions = new ArrayList<IFileRevision>(merged.size());
			for (IBazaarLogMessage mergeLog : merged) {
				revisions.add(buildRevision(mergeLog));
			}
			return revisions.toArray(new IFileRevision[revisions.size()]);
		} else {
			return new IFileRevision[]{revisionMap.get(bazaarRevision.getIBazaarLogMessage().getRevision())};
		}
	}

	public IFileRevision getFileRevision(String id) {
		System.out.println("BazaarFileHistory::getFileRevision(" + id + ")");
		return null;
	}

	public IFileRevision[] getFileRevisions() {
		return revisionMap.values().toArray(new IFileRevision[0]);
	}

	public IFileRevision[] getTargets(IFileRevision revision) {
		System.out.println("BazaarFileHistory::getTargets()");
		return null;
	}

	protected List<IBazaarLogMessage> getLogMessages(IBazaarRevisionSpec revision, boolean limit) throws CoreException {
		return EclipseBazaarCore.getDefault().getLogCacheManager().getLogs(resource, revision, limit);
	}

	protected boolean isBazaarProvider() {
		RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject());
		return provider != null && provider instanceof BazaarTeamProvider;
	}

	public void refresh(IProgressMonitor monitor, boolean limitLogs) throws CoreException {
		if (isBazaarProvider()) {
			revisionMap.clear();
			getLogs(monitor, null, limitLogs);
		}
	}

	public void loadMore(IProgressMonitor monitor) throws CoreException {
		if (isBazaarProvider() && lastLoadedRevision != null) {
			BazaarRevisionRange range = BazaarRevisionRange.getRange(null, BazaarRevision.getRevision(Prefix.BEFORE, lastLoadedRevision.getValue()));
			getLogs(monitor, range, true);
		}
	}

	private void getLogs(IProgressMonitor monitor, IBazaarRevisionSpec rev, boolean limitLogs) throws CoreException {
		monitor.subTask("Fetching log messages...");
		List<IBazaarLogMessage> logs = getLogMessages(rev, limitLogs);
		monitor.worked(500);
		addRevisions(logs);
		monitor.worked(300);
	}

	private void addRevisions(List<IBazaarLogMessage> logMessages) {
		if (logMessages != null) {
			for (IBazaarLogMessage log : logMessages) {
				revisionMap.put(log.getRevision(), buildRevision(log));
				lastLoadedRevision = log.getRevision();
			}
		}
	}

	protected BazaarFileRevision buildRevision(IBazaarLogMessage log) {
		return new BazaarFileRevision(log, resource);
	}

}
